"""Test ingress component client."""

from aioresponses import aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import CreateSessionOptions

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_panels(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test panels API."""
    responses.get(
        f"{SUPERVISOR_URL}/ingress/panels",
        status=200,
        body=load_fixture("ingress_panels.json"),
    )
    result = await supervisor_client.ingress.panels()
    assert "core_ssh" in result
    assert result["core_ssh"].title == "Terminal"
    assert result["core_ssh"].icon == "mdi:console"
    assert result["core_ssh"].admin is True
    assert result["core_ssh"].enable is False
    assert "a0d7b954_vscode" in result
    assert result["a0d7b954_vscode"].title == "Studio Code Server"
    assert result["a0d7b954_vscode"].icon == "mdi:microsoft-visual-studio-code"
    assert result["a0d7b954_vscode"].admin is True
    assert result["a0d7b954_vscode"].enable is True


@pytest.mark.parametrize(
    ("options", "expected_body"),
    [(None, None), (CreateSessionOptions(user_id="test"), {"user_id": "test"})],
)
async def test_create_session(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: CreateSessionOptions | None,
    expected_body: dict[str, str] | None,
) -> None:
    """Test create session API."""
    session_url = f"{SUPERVISOR_URL}/ingress/session"
    responses.post(session_url, status=200, body=load_fixture("create_session.json"))
    result = await supervisor_client.ingress.create_session(options)
    assert result == "abc123"

    assert responses.requests.keys() == {("POST", URL(session_url))}
    session_calls = responses.requests[("POST", URL(session_url))]
    assert len(session_calls) == 1
    assert session_calls[0].kwargs["json"] == expected_body


async def test_validate_session(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test validate session API."""
    validate_url = f"{SUPERVISOR_URL}/ingress/validate_session"
    responses.post(validate_url, status=200)
    assert await supervisor_client.ingress.validate_session("abc123") is None

    assert responses.requests.keys() == {("POST", URL(validate_url))}
    validate_calls = responses.requests[("POST", URL(validate_url))]
    assert len(validate_calls) == 1
    assert validate_calls[0].kwargs["json"] == {"session": "abc123"}



"""Test Home Assistant supervisor client."""

from ipaddress import IPv4Address

from aioresponses import aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import (
    HomeAssistantOptions,
    HomeAssistantRebuildOptions,
    HomeAssistantRestartOptions,
    HomeAssistantStopOptions,
    HomeAssistantUpdateOptions,
)

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_homeassistant_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test Home Assistant info API."""
    responses.get(
        f"{SUPERVISOR_URL}/core/info",
        status=200,
        body=load_fixture("homeassistant_info.json"),
    )
    info = await supervisor_client.homeassistant.info()

    assert info.version == "2024.9.0"
    assert info.update_available is False
    assert info.arch == "aarch64"
    assert info.ssl is False
    assert info.port == 8123
    assert info.audio_input is None
    assert info.audio_output is None
    assert info.ip_address == IPv4Address("172.30.32.1")


async def test_homeassistant_stats(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test Home Assistant stats API."""
    responses.get(
        f"{SUPERVISOR_URL}/core/stats",
        status=200,
        body=load_fixture("homeassistant_stats.json"),
    )
    stats = await supervisor_client.homeassistant.stats()

    assert stats.cpu_percent == 0.01
    assert stats.memory_usage == 678883328
    assert stats.memory_percent == 17.41


async def test_homeassistant_options(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test Home Assistant options API."""
    responses.post(f"{SUPERVISOR_URL}/core/options", status=200)
    assert (
        await supervisor_client.homeassistant.set_options(
            HomeAssistantOptions(watchdog=False, backups_exclude_database=True)
        )
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/core/options"))
    }


@pytest.mark.parametrize("options", [None, HomeAssistantUpdateOptions(backup=False)])
async def test_homeassistant_update(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: HomeAssistantUpdateOptions | None,
) -> None:
    """Test Home Assistant update API."""
    responses.post(f"{SUPERVISOR_URL}/core/update", status=200)
    assert await supervisor_client.homeassistant.update(options) is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/core/update"))}


@pytest.mark.parametrize("options", [None, HomeAssistantRestartOptions(safe_mode=True)])
async def test_homeassistant_restart(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: HomeAssistantRestartOptions | None,
) -> None:
    """Test Home Assistant restart API."""
    responses.post(f"{SUPERVISOR_URL}/core/restart", status=200)
    assert await supervisor_client.homeassistant.restart(options) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/core/restart"))
    }


@pytest.mark.parametrize("options", [None, HomeAssistantStopOptions(force=True)])
async def test_homeassistant_stop(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: HomeAssistantStopOptions | None,
) -> None:
    """Test Home Assistant stop API."""
    responses.post(f"{SUPERVISOR_URL}/core/stop", status=200)
    assert await supervisor_client.homeassistant.stop(options) is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/core/stop"))}


async def test_homeassistant_start(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test Home Assistant start API."""
    responses.post(f"{SUPERVISOR_URL}/core/start", status=200)
    assert await supervisor_client.homeassistant.start() is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/core/start"))}


async def test_homeassistant_check_config(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test Home Assistant check config API."""
    responses.post(f"{SUPERVISOR_URL}/core/check", status=200)
    assert await supervisor_client.homeassistant.check_config() is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/core/check"))}


@pytest.mark.parametrize("options", [None, HomeAssistantRebuildOptions(safe_mode=True)])
async def test_homeassistant_rebuild(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: HomeAssistantRebuildOptions | None,
) -> None:
    """Test Home Assistant rebuild API."""
    responses.post(f"{SUPERVISOR_URL}/core/rebuild", status=200)
    assert await supervisor_client.homeassistant.rebuild(options) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/core/rebuild"))
    }



"""Mounts client for Supervisor."""

from .client import _SupervisorComponentClient
from .models.mounts import CIFSMountRequest, MountsInfo, MountsOptions, NFSMountRequest


class MountsClient(_SupervisorComponentClient):
    """Handle mounts access in supervisor."""

    async def info(self) -> MountsInfo:
        """Get mounts info."""
        result = await self._client.get("mounts")
        return MountsInfo.from_dict(result.data)

    async def options(self, options: MountsOptions) -> None:
        """Set mounts options."""
        await self._client.post("mounts/options", json=options.to_dict())

    async def create_mount(
        self, name: str, config: CIFSMountRequest | NFSMountRequest
    ) -> None:
        """Create a new mount."""
        await self._client.post("mounts", json={"name": name, **config.to_dict()})

    async def update_mount(
        self, name: str, config: CIFSMountRequest | NFSMountRequest
    ) -> None:
        """Update an existing mount."""
        await self._client.put(f"mounts/{name}", json=config.to_dict())

    async def delete_mount(self, name: str) -> None:
        """Delete an existing mount."""
        await self._client.delete(f"mounts/{name}")

    async def reload_mount(self, name: str) -> None:
        """Reload details of an existing mount."""
        await self._client.post(f"mounts/{name}/reload")



"""Host client for supervisor."""

from .client import _SupervisorComponentClient
from .const import TIMEOUT_60_SECONDS
from .models.host import (
    DiskUsage,
    HostInfo,
    HostOptions,
    RebootOptions,
    Service,
    ServiceList,
    ShutdownOptions,
)


class HostClient(_SupervisorComponentClient):
    """Handles host access in supervisor."""

    async def info(self) -> HostInfo:
        """Get host info."""
        result = await self._client.get("host/info")
        return HostInfo.from_dict(result.data)

    async def reboot(self, options: RebootOptions | None = None) -> None:
        """Reboot host."""
        await self._client.post(
            "host/reboot",
            json=options.to_dict() if options else None,
            timeout=TIMEOUT_60_SECONDS,
        )

    async def shutdown(self, options: ShutdownOptions | None = None) -> None:
        """Shutdown host."""
        await self._client.post(
            "host/shutdown", json=options.to_dict() if options else None
        )

    async def reload(self) -> None:
        """Reload host info cache."""
        await self._client.post("host/reload")

    async def set_options(self, options: HostOptions) -> None:
        """Set host options."""
        await self._client.post("host/options", json=options.to_dict())

    async def services(self) -> list[Service]:
        """Get list of available services on host."""
        result = await self._client.get("host/services")
        return ServiceList.from_dict(result.data).services

    async def get_disk_usage(self, max_depth: int = 1) -> DiskUsage:
        """Get disk usage."""
        result = await self._client.get(
            "host/disks/default/usage",
            params={"max_depth": str(max_depth)},
        )
        return DiskUsage.from_dict(result.data)

    # Omitted for now - Log endpoints



"""Tests for aiohasupervisor."""

from pathlib import Path


def get_fixture_path(filename: str) -> Path:
    """Get fixture path."""
    return Path(__package__) / "fixtures" / filename


def load_fixture(filename: str) -> str:
    """Load a fixture."""
    fixture = get_fixture_path(filename)
    return fixture.read_text(encoding="utf-8")



"""Test jobs supervisor client."""

from datetime import datetime
from uuid import UUID

from aioresponses import aioresponses
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import JobCondition, JobsOptions

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_jobs_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test jobs info API."""
    responses.get(
        f"{SUPERVISOR_URL}/jobs/info", status=200, body=load_fixture("jobs_info.json")
    )
    info = await supervisor_client.jobs.info()
    assert info.ignore_conditions == [JobCondition.FREE_SPACE]

    assert info.jobs[0].name == "backup_manager_partial_backup"
    assert info.jobs[0].reference == "89cafa67"
    assert info.jobs[0].uuid.hex == "2febe59311f94d6ba36f6f9f73357ca8"
    assert info.jobs[0].progress == 0
    assert info.jobs[0].stage == "finishing_file"
    assert info.jobs[0].done is True
    assert info.jobs[0].errors == []
    assert info.jobs[0].created == datetime.fromisoformat(
        "2025-01-30T20:55:12.859349+00:00"
    )
    assert info.jobs[0].child_jobs[0].name == "backup_store_folders"
    assert info.jobs[0].child_jobs[0].child_jobs[0].name == "backup_folder_save"
    assert info.jobs[0].child_jobs[0].child_jobs[0].reference == "ssl"
    assert info.jobs[0].child_jobs[0].child_jobs[0].child_jobs == []

    assert info.jobs[1].name == "backup_manager_partial_restore"
    assert info.jobs[1].reference == "cfddca18"
    assert info.jobs[1].errors[0].type == "BackupInvalidError"
    assert info.jobs[1].errors[0].message == "Invalid password for backup cfddca18"
    assert info.jobs[1].errors[0].stage == "restore_backup"


async def test_jobs_set_options(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test jobs set options API."""
    responses.post(f"{SUPERVISOR_URL}/jobs/options", status=200)
    assert (
        await supervisor_client.jobs.set_options(
            JobsOptions(ignore_conditions=[JobCondition.FREE_SPACE])
        )
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/jobs/options"))
    }


async def test_jobs_reset(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test jobs reset API."""
    responses.post(f"{SUPERVISOR_URL}/jobs/reset", status=200)
    assert await supervisor_client.jobs.reset() is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/jobs/reset"))}


async def test_jobs_get_job(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test jobs get job API."""
    responses.get(
        f"{SUPERVISOR_URL}/jobs/2febe59311f94d6ba36f6f9f73357ca8",
        status=200,
        body=load_fixture("jobs_get_job.json"),
    )
    info = await supervisor_client.jobs.get_job(
        UUID("2febe59311f94d6ba36f6f9f73357ca8")
    )

    assert info.name == "backup_manager_partial_backup"
    assert info.reference == "89cafa67"
    assert info.uuid.hex == "2febe59311f94d6ba36f6f9f73357ca8"
    assert info.progress == 0
    assert info.stage == "finishing_file"
    assert info.done is True
    assert info.errors == []
    assert info.created == datetime.fromisoformat("2025-01-30T20:55:12.859349+00:00")
    assert info.child_jobs[0].name == "backup_store_folders"
    assert info.child_jobs[0].child_jobs[0].name == "backup_folder_save"
    assert info.child_jobs[0].child_jobs[0].reference == "ssl"
    assert info.child_jobs[0].child_jobs[0].child_jobs == []


async def test_jobs_get_job_extra(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test jobs get job API with extra metadata."""
    responses.get(
        f"{SUPERVISOR_URL}/jobs/2febe59311f94d6ba36f6f9f73357ca8",
        status=200,
        body=load_fixture("jobs_get_job_extra.json"),
    )
    info = await supervisor_client.jobs.get_job(
        UUID("2febe59311f94d6ba36f6f9f73357ca8")
    )

    assert info.name == "docker_interface_install"
    assert info.reference == "homeassistant"
    assert info.child_jobs[0].name == "Pulling container image layer"
    assert info.child_jobs[0].reference == "1e214cd6d7d0"
    assert info.child_jobs[0].progress == 26.1
    assert info.child_jobs[0].stage == "Downloading"
    assert info.child_jobs[0].extra == {"current": 227726144, "total": 436480882}


async def test_jobs_delete_job(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test jobs delete job API."""
    responses.delete(
        f"{SUPERVISOR_URL}/jobs/2febe59311f94d6ba36f6f9f73357ca8", status=200
    )
    assert (
        await supervisor_client.jobs.delete_job(
            UUID("2febe59311f94d6ba36f6f9f73357ca8")
        )
        is None
    )
    assert responses.requests.keys() == {
        ("DELETE", URL(f"{SUPERVISOR_URL}/jobs/2febe59311f94d6ba36f6f9f73357ca8"))
    }


async def test_jobs_info_backward_compatibility_no_stage(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test jobs info API with error lacking stage field for backward compatibility."""
    responses.get(
        f"{SUPERVISOR_URL}/jobs/info",
        status=200,
        body=load_fixture("jobs_info_no_stage.json"),
    )
    info = await supervisor_client.jobs.info()
    assert info.ignore_conditions == [JobCondition.FREE_SPACE]

    assert info.jobs[0].name == "test_job_error_no_stage"
    assert info.jobs[0].reference == "test123"
    assert info.jobs[0].errors[0].type == "TestError"
    assert info.jobs[0].errors[0].message == "Test error without stage field"
    assert info.jobs[0].errors[0].stage is None



"""Test for supervisor management client."""

from ipaddress import IPv4Address

from aioresponses import aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import SupervisorOptions, SupervisorUpdateOptions
from aiohasupervisor.models.supervisor import DetectBlockingIO

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_supervisor_ping(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test supervisor ping API."""
    responses.get(f"{SUPERVISOR_URL}/supervisor/ping", status=200)
    assert await supervisor_client.supervisor.ping() is None
    assert responses.requests.keys() == {
        ("GET", URL(f"{SUPERVISOR_URL}/supervisor/ping"))
    }


async def test_supervisor_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test supervisor info API."""
    responses.get(
        f"{SUPERVISOR_URL}/supervisor/info",
        status=200,
        body=load_fixture("supervisor_info.json"),
    )
    info = await supervisor_client.supervisor.info()

    assert info.version == "2024.09.1"
    assert info.channel == "stable"
    assert info.arch == "aarch64"
    assert info.supported is True
    assert info.healthy is True
    assert info.logging == "info"
    assert info.ip_address == IPv4Address("172.30.32.2")
    assert info.country is None
    assert info.detect_blocking_io is False


@pytest.mark.parametrize("field", ["version_latest", "arch"])
async def test_supervisor_info_optional_fields_none(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    field: str,
) -> None:
    """Test supervisor info API when optional fields are None."""
    import json

    fixture = json.loads(load_fixture("supervisor_info.json"))
    fixture["data"][field] = None
    responses.get(
        f"{SUPERVISOR_URL}/supervisor/info",
        status=200,
        payload=fixture,
    )
    info = await supervisor_client.supervisor.info()
    assert getattr(info, field) is None


async def test_supervisor_stats(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test supervisor stats API."""
    responses.get(
        f"{SUPERVISOR_URL}/supervisor/stats",
        status=200,
        body=load_fixture("supervisor_stats.json"),
    )
    stats = await supervisor_client.supervisor.stats()

    assert stats.cpu_percent == 0.04
    assert stats.memory_usage == 243982336
    assert stats.memory_limit == 3899138048
    assert stats.memory_percent == 6.26


@pytest.mark.parametrize(
    "options", [None, SupervisorUpdateOptions(version="2024.01.0")]
)
async def test_supervisor_update(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: SupervisorUpdateOptions | None,
) -> None:
    """Test supervisor update API."""
    responses.post(f"{SUPERVISOR_URL}/supervisor/update", status=200)
    assert await supervisor_client.supervisor.update(options) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/supervisor/update"))
    }


async def test_supervisor_reload(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test supervisor reload API."""
    responses.post(f"{SUPERVISOR_URL}/supervisor/reload", status=200)
    assert await supervisor_client.supervisor.reload() is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/supervisor/reload"))
    }


async def test_supervisor_restart(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test supervisor restart API."""
    responses.post(f"{SUPERVISOR_URL}/supervisor/restart", status=200)
    assert await supervisor_client.supervisor.restart() is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/supervisor/restart"))
    }


async def test_supervisor_options(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test supervisor options API."""
    responses.post(f"{SUPERVISOR_URL}/supervisor/options", status=200)
    assert (
        await supervisor_client.supervisor.set_options(
            SupervisorOptions(
                debug=True,
                debug_block=True,
                country="NL",
                detect_blocking_io=DetectBlockingIO.ON_AT_STARTUP,
            )
        )
        is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/supervisor/options"))
    }
    assert (
        request := responses.requests[
            ("POST", URL(f"{SUPERVISOR_URL}/supervisor/options"))
        ]
    )
    assert request[0].kwargs["json"] == {
        "debug": True,
        "debug_block": True,
        "country": "NL",
        "detect_blocking_io": "on_at_startup",
    }


async def test_supervisor_repair(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test supervisor repair API."""
    responses.post(f"{SUPERVISOR_URL}/supervisor/repair", status=200)
    assert await supervisor_client.supervisor.repair() is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/supervisor/repair"))
    }



"""Models for ingress APIs."""

from dataclasses import dataclass

from .base import Request, ResponseData


@dataclass(frozen=True, slots=True)
class IngressPanel(ResponseData):
    """IngressPanel model."""

    title: str
    icon: str
    admin: bool
    enable: bool | None


@dataclass(frozen=True, slots=True)
class IngressPanels(ResponseData):
    """IngressPanels model."""

    panels: dict[str, IngressPanel]


@dataclass(frozen=True, slots=True)
class CreateSessionOptions(Request):
    """CreateSessionOptions model."""

    user_id: str


@dataclass(frozen=True, slots=True)
class Session(ResponseData):
    """Session model."""

    session: str



"""Jobs client for supervisor."""

from uuid import UUID

from .client import _SupervisorComponentClient
from .models.jobs import Job, JobsInfo, JobsOptions


class JobsClient(_SupervisorComponentClient):
    """Handles Jobs access in Supervisor."""

    async def info(self) -> JobsInfo:
        """Get Jobs info."""
        result = await self._client.get("jobs/info")
        return JobsInfo.from_dict(result.data)

    async def set_options(self, options: JobsOptions) -> None:
        """Set Jobs options."""
        await self._client.post("jobs/options", json=options.to_dict())

    async def reset(self) -> None:
        """Reset Jobs options (primarily clears previously ignored job conditions)."""
        await self._client.post("jobs/reset")

    async def get_job(self, job: UUID) -> Job:
        """Get details of a job."""
        result = await self._client.get(f"jobs/{job.hex}")
        return Job.from_dict(result.data)

    async def delete_job(self, job: UUID) -> None:
        """Remove a done job from Supervisor's cache."""
        await self._client.delete(f"jobs/{job.hex}")



"""Test host supervisor client."""

from datetime import UTC, datetime

from aioresponses import aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import HostOptions, RebootOptions, ShutdownOptions

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_host_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test host info API."""
    responses.get(
        f"{SUPERVISOR_URL}/host/info", status=200, body=load_fixture("host_info.json")
    )
    result = await supervisor_client.host.info()
    assert result.agent_version == "1.6.0"
    assert result.chassis == "embedded"
    assert result.virtualization == ""
    assert result.disk_total == 27.9
    assert result.disk_life_time == 10
    assert result.features == [
        "reboot",
        "shutdown",
        "services",
        "network",
        "hostname",
        "timedate",
        "os_agent",
        "haos",
        "resolved",
        "journal",
        "disk",
        "mount",
    ]
    assert result.hostname == "homeassistant"
    assert result.llmnr_hostname == "homeassistant3"
    assert result.dt_utc == datetime(2024, 10, 3, 0, 0, 0, 0, UTC)
    assert result.dt_synchronized is True
    assert result.startup_time == 1.966311


@pytest.mark.parametrize("options", [None, RebootOptions(force=True)])
async def test_host_reboot(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: RebootOptions | None,
) -> None:
    """Test host reboot API."""
    responses.post(f"{SUPERVISOR_URL}/host/reboot", status=200)
    assert await supervisor_client.host.reboot(options) is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/host/reboot"))}


@pytest.mark.parametrize("options", [None, ShutdownOptions(force=True)])
async def test_host_shutdown(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: ShutdownOptions | None,
) -> None:
    """Test host shutdown API."""
    responses.post(f"{SUPERVISOR_URL}/host/shutdown", status=200)
    assert await supervisor_client.host.shutdown(options) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/host/shutdown"))
    }


async def test_host_reload(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test host reload API."""
    responses.post(f"{SUPERVISOR_URL}/host/reload", status=200)
    assert await supervisor_client.host.reload() is None
    assert responses.requests.keys() == {("POST", URL(f"{SUPERVISOR_URL}/host/reload"))}


async def test_host_options(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test host options API."""
    responses.post(f"{SUPERVISOR_URL}/host/options", status=200)
    assert (
        await supervisor_client.host.set_options(HostOptions(hostname="test")) is None
    )
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/host/options"))
    }


async def test_host_services(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test host services API."""
    responses.get(
        f"{SUPERVISOR_URL}/host/services",
        status=200,
        body=load_fixture("host_services.json"),
    )
    result = await supervisor_client.host.services()
    assert result[0].name == "emergency.service"
    assert result[0].description == "Emergency Shell"
    assert result[0].state == "inactive"
    assert result[-1].name == "systemd-resolved.service"
    assert result[-1].description == "Network Name Resolution"
    assert result[-1].state == "active"


async def test_host_disk_usage(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test host disk usage API."""
    responses.get(
        f"{SUPERVISOR_URL}/host/disks/default/usage?max_depth=1",
        status=200,
        body=load_fixture("host_disk_usage.json"),
    )
    result = await supervisor_client.host.get_disk_usage()

    # Test top-level properties
    assert result.total_bytes == 503312781312
    assert result.used_bytes == 430243422208
    assert result.children is not None

    # Test children structure
    children = result.children
    assert children is not None
    assert len(children) == 8  # Should have 8 children

    # Find specific children by id
    assert (
        addons_data := next(child for child in children if child.id == "addons_data")
    )
    assert next(child for child in children if child.id == "addons_config")
    assert next(child for child in children if child.id == "media")
    assert next(child for child in children if child.id == "share")
    assert (backup := next(child for child in children if child.id == "backup"))
    assert next(child for child in children if child.id == "ssl")
    assert (
        homeassistant := next(
            child for child in children if child.id == "homeassistant"
        )
    )
    assert next(child for child in children if child.id == "system")

    # Test nested children (recursive structure)
    assert addons_data.used_bytes == 42347618720
    assert addons_data.children is not None
    assert len(addons_data.children) == 4

    # Find specific nested children
    assert next(
        child for child in addons_data.children if child.id == "77f1785d_remote_api"
    )
    assert next(child for child in addons_data.children if child.id == "core_samba")
    assert (
        plex_addon := next(
            child for child in addons_data.children if child.id == "a0d7b954_plex"
        )
    )
    assert next(child for child in addons_data.children if child.id == "core_whisper")

    # Test deeper nesting
    assert plex_addon.used_bytes == 757750613
    assert plex_addon.children is None  # Leaf node

    # Test another branch
    assert homeassistant.used_bytes == 444089236
    assert homeassistant.children is not None
    assert len(homeassistant.children) == 3

    # Find specific homeassistant children
    assert next(child for child in homeassistant.children if child.id == "image")
    assert next(
        child for child in homeassistant.children if child.id == "custom_components"
    )
    assert next(child for child in homeassistant.children if child.id == "www")

    # Test leaf node without children
    assert backup.used_bytes == 268350699520
    assert backup.children is None


async def test_host_disk_usage_with_custom_depth(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test host disk usage API with custom max_depth."""
    responses.get(
        f"{SUPERVISOR_URL}/host/disks/default/usage?max_depth=3",
        status=200,
        body=load_fixture("host_disk_usage.json"),
    )
    result = await supervisor_client.host.get_disk_usage(max_depth=3)

    # Test that the custom max_depth parameter was used
    assert result.total_bytes == 503312781312
    assert result.used_bytes == 430243422208



"""Tests for store supervisor client."""

from json import loads

from aioresponses import aioresponses
import pytest
from yarl import URL

from aiohasupervisor import SupervisorClient
from aiohasupervisor.exceptions import (
    AddonNotSupportedArchitectureError,
    AddonNotSupportedHomeAssistantVersionError,
    AddonNotSupportedMachineTypeError,
    SupervisorBadRequestError,
    SupervisorError,
)
from aiohasupervisor.models import StoreAddonUpdate, StoreAddRepository
from aiohasupervisor.models.addons import StoreAddonInstall

from . import load_fixture
from .const import SUPERVISOR_URL


async def test_store_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store info API."""
    responses.get(
        f"{SUPERVISOR_URL}/store", status=200, body=load_fixture("store_info.json")
    )
    store = await supervisor_client.store.info()

    assert len(store.repositories) == 6
    assert store.repositories[0].slug == "local"
    assert store.repositories[0].name == "Local add-ons"

    assert len(store.addons) == 88
    assert store.addons[0].slug == "d5369777_music_assistant"
    assert store.addons[0].name == "Music Assistant Server"


async def test_store_addons_list(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addons list API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons",
        status=200,
        body=load_fixture("store_addons_list.json"),
    )
    addons = await supervisor_client.store.addons_list()

    assert len(addons) == 88
    assert addons[0].slug == "d5369777_music_assistant"
    assert addons[0].name == "Music Assistant Server"


async def test_store_repositories_list(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store repositories list API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/repositories",
        status=200,
        body=load_fixture("store_repositories_list.json"),
    )
    repositories = await supervisor_client.store.repositories_list()

    assert len(repositories) == 6
    assert repositories[0].slug == "local"
    assert repositories[0].name == "Local add-ons"


async def test_store_addon_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon info API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto",
        status=200,
        body=load_fixture("store_addon_info.json"),
    )
    info = await supervisor_client.store.addon_info("core_mosquitto")

    assert info.name == "Mosquitto broker"
    assert info.slug == "core_mosquitto"
    assert info.signed is True
    assert info.supervisor_api is False
    assert info.supervisor_role == "default"


async def test_store_addon_changelog(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon info changelog API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto/changelog",
        status=200,
        body=load_fixture("store_addon_changelog.txt"),
    )
    changelog = await supervisor_client.store.addon_changelog("core_mosquitto")
    assert changelog.startswith("# Changelog")


async def test_store_addon_documentation(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon documentation API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto/documentation",
        status=200,
        body=load_fixture("store_addon_documentation.txt"),
    )
    documentation = await supervisor_client.store.addon_documentation("core_mosquitto")
    assert documentation.startswith("# Home Assistant Add-on: Mosquitto broker")


@pytest.mark.parametrize(
    ("options", "has_timeout"),
    [
        (None, False),
        (StoreAddonInstall(), False),
        (StoreAddonInstall(background=False), False),
        (StoreAddonInstall(background=True), True),
    ],
)
async def test_store_addon_install(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: StoreAddonInstall | None,
    has_timeout: bool,  # noqa: FBT001
) -> None:
    """Test store addon install API."""
    responses.post(f"{SUPERVISOR_URL}/store/addons/core_mosquitto/install", status=200)

    assert (
        await supervisor_client.store.install_addon("core_mosquitto", options)
    ) is None
    assert (
        bool(
            responses.requests[
                ("POST", URL(f"{SUPERVISOR_URL}/store/addons/core_mosquitto/install"))
            ][0].kwargs["timeout"]
        )
        is has_timeout
    )


@pytest.mark.parametrize(
    ("options", "has_timeout"),
    [
        (None, False),
        (StoreAddonUpdate(), False),
        (StoreAddonUpdate(backup=True), False),
        (StoreAddonUpdate(background=False), False),
        (StoreAddonUpdate(background=True), True),
    ],
)
async def test_store_addon_update(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    options: StoreAddonUpdate | None,
    has_timeout: bool,  # noqa: FBT001
) -> None:
    """Test store addon update API."""
    responses.post(f"{SUPERVISOR_URL}/store/addons/core_mosquitto/update", status=200)

    assert (
        await supervisor_client.store.update_addon("core_mosquitto", options)
    ) is None
    assert (
        bool(
            responses.requests[
                ("POST", URL(f"{SUPERVISOR_URL}/store/addons/core_mosquitto/update"))
            ][0].kwargs["timeout"]
        )
        is has_timeout
    )


async def test_store_addon_availability(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store addon availability API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto/availability", status=200
    )

    assert (await supervisor_client.store.addon_availability("core_mosquitto")) is None


@pytest.mark.parametrize(
    ("error_fixture", "error_key", "exc_type"),
    [
        (
            "store_addon_availability_error_architecture.json",
            "addon_not_supported_architecture_error",
            AddonNotSupportedArchitectureError,
        ),
        (
            "store_addon_availability_error_machine.json",
            "addon_not_supported_machine_type_error",
            AddonNotSupportedMachineTypeError,
        ),
        (
            "store_addon_availability_error_home_assistant.json",
            "addon_not_supported_home_assistant_version_error",
            AddonNotSupportedHomeAssistantVersionError,
        ),
        (
            "store_addon_availability_error_other.json",
            None,
            SupervisorBadRequestError,
        ),
    ],
)
async def test_store_addon_availability_error(
    responses: aioresponses,
    supervisor_client: SupervisorClient,
    error_fixture: str,
    error_key: str | None,
    exc_type: type[SupervisorError],
) -> None:
    """Test store addon availability errors."""
    error_body = load_fixture(error_fixture)
    error_data = loads(error_body)

    def check_availability_error(err: SupervisorError) -> bool:
        assert err.error_key == error_key
        assert err.extra_fields == error_data["extra_fields"]
        return True

    # Availability API
    responses.get(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto/availability",
        status=400,
        body=error_body,
    )
    with pytest.raises(
        exc_type, match=error_data["message"], check=check_availability_error
    ):
        await supervisor_client.store.addon_availability("core_mosquitto")

    # Install API
    responses.post(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto/install",
        status=400,
        body=error_body,
    )
    with pytest.raises(
        exc_type, match=error_data["message"], check=check_availability_error
    ):
        await supervisor_client.store.install_addon("core_mosquitto")

    # Update API
    responses.post(
        f"{SUPERVISOR_URL}/store/addons/core_mosquitto/update",
        status=400,
        body=error_body,
    )
    with pytest.raises(
        exc_type, match=error_data["message"], check=check_availability_error
    ):
        await supervisor_client.store.update_addon("core_mosquitto")


async def test_store_reload(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store reload API."""
    responses.post(f"{SUPERVISOR_URL}/store/reload", status=200)

    assert (await supervisor_client.store.reload()) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/store/reload"))
    }


async def test_store_repository_info(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store repository info API."""
    responses.get(
        f"{SUPERVISOR_URL}/store/repositories/core",
        status=200,
        body=load_fixture("store_repository_info.json"),
    )
    repository = await supervisor_client.store.repository_info("core")
    assert repository.slug == "core"
    assert repository.name == "Official add-ons"
    assert repository.source == "core"


async def test_store_add_repository(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store add repository API."""
    responses.post(f"{SUPERVISOR_URL}/store/repositories", status=200)

    assert (
        await supervisor_client.store.add_repository(
            StoreAddRepository(repository="test")
        )
    ) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/store/repositories"))
    }


async def test_store_remove_repository(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store remove repository API."""
    responses.delete(f"{SUPERVISOR_URL}/store/repositories/test", status=200)

    assert (await supervisor_client.store.remove_repository("test")) is None
    assert responses.requests.keys() == {
        ("DELETE", URL(f"{SUPERVISOR_URL}/store/repositories/test"))
    }


async def test_store_repair_repository(
    responses: aioresponses, supervisor_client: SupervisorClient
) -> None:
    """Test store repository repair API."""
    responses.post(f"{SUPERVISOR_URL}/store/repositories/test/repair", status=200)

    assert (await supervisor_client.store.repair_repository("test")) is None
    assert responses.requests.keys() == {
        ("POST", URL(f"{SUPERVISOR_URL}/store/repositories/test/repair"))
    }



"""Simple connectivity test."""

import asyncio
import json
import os
from typing import final

from aiohasupervisor import SupervisorClient
from aiohasupervisor.models import RootInfo

SUPERVISOR_API_URL: final = os.environ.get("SUPERVISOR_API_URL")
SUPERVISOR_TOKEN: final = os.environ.get("SUPERVISOR_TOKEN")

if not SUPERVISOR_API_URL:
    raise RuntimeError("SUPERVISOR_API_URL env must be set")
if not SUPERVISOR_TOKEN:
    raise RuntimeError("SUPERVISOR_TOKEN env must be set")

client = SupervisorClient(SUPERVISOR_API_URL, SUPERVISOR_TOKEN)


async def get_info() -> RootInfo:
    """Get root info."""
    async with client:
        return await client.info()


info = asyncio.run(get_info())
print(json.dumps(info.to_dict(), indent=4))  # noqa: T201



"""Main client for supervisor."""

from typing import Self

from aiohttp import ClientSession, ClientTimeout

from .addons import AddonsClient
from .backups import BackupsClient
from .client import _SupervisorClient
from .discovery import DiscoveryClient
from .homeassistant import HomeAssistantClient
from .host import HostClient
from .ingress import IngressClient
from .jobs import JobsClient
from .models.root import AvailableUpdate, AvailableUpdates, RootInfo
from .mounts import MountsClient
from .network import NetworkClient
from .os import OSClient
from .resolution import ResolutionClient
from .store import StoreClient
from .supervisor import SupervisorManagementClient


class SupervisorClient:
    """Main supervisor client for all Supervisor access."""

    def __init__(
        self,
        api_host: str,
        token: str,
        session: ClientSession | None = None,
    ) -> None:
        """Initialize client."""
        self._client = _SupervisorClient(api_host, token, session)
        self._addons = AddonsClient(self._client)
        self._os = OSClient(self._client)
        self._backups = BackupsClient(self._client)
        self._discovery = DiscoveryClient(self._client)
        self._jobs = JobsClient(self._client)
        self._mounts = MountsClient(self._client)
        self._network = NetworkClient(self._client)
        self._host = HostClient(self._client)
        self._resolution = ResolutionClient(self._client)
        self._store = StoreClient(self._client)
        self._supervisor = SupervisorManagementClient(self._client)
        self._homeassistant = HomeAssistantClient(self._client)
        self._ingress = IngressClient(self._client)

    @property
    def addons(self) -> AddonsClient:
        """Get addons component client."""
        return self._addons

    @property
    def homeassistant(self) -> HomeAssistantClient:
        """Get Home Assistant component client."""
        return self._homeassistant

    @property
    def os(self) -> OSClient:
        """Get OS component client."""
        return self._os

    @property
    def backups(self) -> BackupsClient:
        """Get backups component client."""
        return self._backups

    @property
    def discovery(self) -> DiscoveryClient:
        """Get discovery component client."""
        return self._discovery

    @property
    def jobs(self) -> JobsClient:
        """Get jobs component client."""
        return self._jobs

    @property
    def mounts(self) -> MountsClient:
        """Get mounts component client."""
        return self._mounts

    @property
    def network(self) -> NetworkClient:
        """Get network component client."""
        return self._network

    @property
    def host(self) -> HostClient:
        """Get host component client."""
        return self._host

    @property
    def resolution(self) -> ResolutionClient:
        """Get resolution center component client."""
        return self._resolution

    @property
    def store(self) -> StoreClient:
        """Get store component client."""
        return self._store

    @property
    def supervisor(self) -> SupervisorManagementClient:
        """Get supervisor component client."""
        return self._supervisor

    @property
    def ingress(self) -> IngressClient:
        """Get ingress component client."""
        return self._ingress

    async def info(self) -> RootInfo:
        """Get root info."""
        result = await self._client.get("info")
        return RootInfo.from_dict(result.data)

    async def reload_updates(self) -> None:
        """Reload updates.

        Reload main components update information (OS, Supervisor, Core and plug-ins).
        """
        await self._client.post("reload_updates", timeout=ClientTimeout(total=300))

    async def refresh_updates(self) -> None:
        """Refresh updates.

        Discouraged as this endpoint does two things at once. Use the `reload_updates()`
        and `store.reload()` instead.
        """
        await self._client.post("refresh_updates", timeout=ClientTimeout(total=300))

    async def available_updates(self) -> list[AvailableUpdate]:
        """Get available updates."""
        result = await self._client.get("available_updates")
        return AvailableUpdates.from_dict(result.data).available_updates

    async def close(self) -> None:
        """Close open client session."""
        await self._client.close()

    async def __aenter__(self) -> Self:
        """Async enter, closes session on exit."""
        return self

    async def __aexit__(self, *_exc_info: object) -> None:
        """Close session."""
        await self.close()



"""Ingress client for supervisor."""

from .client import _SupervisorComponentClient
from .const import ResponseType
from .models.ingress import CreateSessionOptions, IngressPanel, IngressPanels, Session


class IngressClient(_SupervisorComponentClient):
    """Handles ingress access in Supervisor.

    This includes only the APIs with fixed paths and models from Supervisor's Ingress
    API. The wildcard proxy endpoints that allow the UI to talk to addons through Core
    and Supervisor are intentionally omitted as they can't be modeled.
    """

    async def panels(self) -> dict[str, IngressPanel]:
        """Get ingress panels, returns a map of addon slug to panel info."""
        result = await self._client.get("ingress/panels")
        return IngressPanels.from_dict(result.data).panels

    async def create_session(self, options: CreateSessionOptions | None = None) -> str:
        """Create a new ingress session."""
        result = await self._client.post(
            "ingress/session",
            json=options.to_dict() if options else None,
            response_type=ResponseType.JSON,
        )
        return Session.from_dict(result.data).session

    async def validate_session(self, session: str) -> None:
        """Validate an existing ingress session."""
        body = Session(session=session).to_dict()
        await self._client.post("ingress/validate_session", json=body)



"""Models for supervisor network."""

from abc import ABC
from dataclasses import dataclass
from enum import StrEnum
from ipaddress import (
    IPv4Address,
    IPv4Interface,
    IPv4Network,
    IPv6Address,
    IPv6Interface,
)

from .base import Options, Request, ResponseData

# --- ENUMS ----


class InterfaceType(StrEnum):
    """InterfaceType type."""

    ETHERNET = "ethernet"
    WIRELESS = "wireless"
    VLAN = "vlan"


class InterfaceMethod(StrEnum):
    """InterfaceMethod type."""

    DISABLED = "disabled"
    STATIC = "static"
    AUTO = "auto"


class InterfaceAddrGenMode(StrEnum):
    """IPv6 address generation mode."""

    EUI64 = "eui64"
    STABLE_PRIVACY = "stable-privacy"
    DEFAULT_OR_EUI64 = "default-or-eui64"
    DEFAULT = "default"


class InterfaceIp6Privacy(StrEnum):
    """IPv6 privacy extension behavior."""

    DEFAULT = "default"
    DISABLED = "disabled"
    ENABLED_PREFER_PUBLIC = "enabled-prefer-public"
    ENABLED = "enabled"


class WifiMode(StrEnum):
    """WifiMode type."""

    INFRASTRUCTURE = "infrastructure"
    MESH = "mesh"
    ADHOC = "adhoc"
    AP = "ap"


class AuthMethod(StrEnum):
    """AuthMethod type."""

    OPEN = "open"
    WEP = "wep"
    WPA_PSK = "wpa-psk"


class MulticastDnsMode(StrEnum):
    """Multicast DNS working mode."""

    DEFAULT = "default"
    OFF = "off"
    RESOLVE = "resolve"
    ANNOUNCE = "announce"


# --- OBJECTS ----


@dataclass(frozen=True)
class IpBase(ABC):
    """IpBase ABC type."""

    method: InterfaceMethod
    ready: bool | None


@dataclass(frozen=True, slots=True)
class IPv4(IpBase, ResponseData):
    """IPv4 model."""

    address: list[IPv4Interface]
    nameservers: list[IPv4Address]
    gateway: IPv4Address | None
    route_metric: int | None


@dataclass(frozen=True, slots=True)
class IPv6(IpBase, ResponseData):
    """IPv6 model."""

    address: list[IPv6Interface]
    nameservers: list[IPv6Address]
    gateway: IPv6Address | None
    route_metric: int | None
    addr_gen_mode: InterfaceAddrGenMode
    ip6_privacy: InterfaceIp6Privacy


@dataclass(frozen=True, slots=True)
class Wifi(ResponseData):
    """Wifi model."""

    mode: WifiMode
    auth: AuthMethod
    ssid: str
    signal: int | None


@dataclass(frozen=True, slots=True)
class Vlan(ResponseData):
    """Vlan model."""

    id: int
    parent: str | None


@dataclass(frozen=True, slots=True)
class NetworkInterface(ResponseData):
    """NetworkInterface model."""

    interface: str
    type: InterfaceType
    enabled: bool
    connected: bool
    primary: bool
    mac: str
    ipv4: IPv4 | None
    ipv6: IPv6 | None
    wifi: Wifi | None
    vlan: Vlan | None
    mdns: MulticastDnsMode | None
    llmnr: MulticastDnsMode | None


@dataclass(frozen=True, slots=True)
class DockerNetwork(ResponseData):
    """DockerNetwork model."""

    interface: str
    address: IPv4Network
    gateway: IPv4Address
    dns: IPv4Address


@dataclass(frozen=True, slots=True)
class NetworkInfo(ResponseData):
    """NetworkInfo model."""

    interfaces: list[NetworkInterface]
    docker: DockerNetwork
    host_internet: bool | None
    supervisor_internet: bool


@dataclass(frozen=True, slots=True)
class IPv4Config(Request):
    """IPv4Config model."""

    address: list[IPv4Interface] | None = None
    method: InterfaceMethod | None = None
    gateway: IPv4Address | None = None
    nameservers: list[IPv4Address] | None = None
    route_metric: int | None = None


@dataclass(frozen=True, slots=True)
class IPv6Config(Request):
    """IPv6Config model."""

    address: list[IPv6Interface] | None = None
    method: InterfaceMethod | None = None
    gateway: IPv6Address | None = None
    nameservers: list[IPv6Address] | None = None
    route_metric: int | None = None
    addr_gen_mode: InterfaceAddrGenMode | None = None
    ip6_privacy: InterfaceIp6Privacy | None = None


@dataclass(frozen=True, slots=True)
class WifiConfig(Request):
    """WifiConfig model."""

    mode: WifiMode | None = None
    method: AuthMethod | None = None
    ssid: str | None = None
    psk: str | None = None


@dataclass(frozen=True, slots=True)
class NetworkInterfaceConfig(Options):
    """NetworkInterfaceConfig model."""

    ipv4: IPv4Config | None = None
    ipv6: IPv6Config | None = None
    wifi: WifiConfig | None = None
    mdns: MulticastDnsMode | None = None
    llmnr: MulticastDnsMode | None = None
    enabled: bool | None = None


@dataclass(frozen=True, slots=True)
class AccessPoint(ResponseData):
    """AccessPoint model."""

    mode: WifiMode
    ssid: str
    frequency: int
    signal: int
    mac: str


@dataclass(frozen=True, slots=True)
class AccessPointList(ResponseData):
    """AccessPointList model."""

    accesspoints: list[AccessPoint]


@dataclass(frozen=True, slots=True)
class VlanConfig(Options):
    """VlanConfig model."""

    ipv4: IPv4Config | None = None
    ipv6: IPv6Config | None = None
    mdns: MulticastDnsMode | None = None
    llmnr: MulticastDnsMode | None = None



"""Addons client for Supervisor."""

from typing import Any

from .client import _SupervisorComponentClient
from .const import TIMEOUT_60_SECONDS, ResponseType
from .models.addons import (
    AddonsConfigValidate,
    AddonsList,
    AddonsOptions,
    AddonsRebuild,
    AddonsSecurityOptions,
    AddonsStats,
    AddonsUninstall,
    InstalledAddon,
    InstalledAddonComplete,
)


class AddonsClient(_SupervisorComponentClient):
    """Handles installed addon access in Supervisor."""

    async def list(self) -> list[InstalledAddon]:
        """Get installed addons."""
        result = await self._client.get("addons")
        return AddonsList.from_dict(result.data).addons

    async def addon_info(self, addon: str) -> InstalledAddonComplete:
        """Get all info for addon."""
        result = await self._client.get(f"addons/{addon}/info")
        return InstalledAddonComplete.from_dict(result.data)

    async def uninstall_addon(
        self,
        addon: str,
        options: AddonsUninstall | None = None,
    ) -> None:
        """Uninstall an addon."""
        await self._client.post(
            f"addons/{addon}/uninstall",
            json=options.to_dict() if options else None,
            timeout=TIMEOUT_60_SECONDS,
        )

    async def start_addon(self, addon: str) -> None:
        """Start an addon."""
        await self._client.post(f"addons/{addon}/start", timeout=TIMEOUT_60_SECONDS)

    async def stop_addon(self, addon: str) -> None:
        """Stop an addon."""
        await self._client.post(f"addons/{addon}/stop", timeout=TIMEOUT_60_SECONDS)

    async def restart_addon(self, addon: str) -> None:
        """Restart an addon."""
        await self._client.post(f"addons/{addon}/restart", timeout=None)

    async def set_addon_options(self, addon: str, options: AddonsOptions) -> None:
        """Set options for addon."""
        await self._client.post(f"addons/{addon}/options", json=options.to_dict())

    async def addon_config_validate(
        self,
        addon: str,
        config: dict[str, Any],
    ) -> AddonsConfigValidate:
        """Validate config for an addon."""
        result = await self._client.post(
            f"addons/{addon}/options/validate",
            response_type=ResponseType.JSON,
            json=config,
        )
        return AddonsConfigValidate.from_dict(result.data)

    async def addon_config(self, addon: str) -> dict[str, Any]:
        """Get config for addon."""
        result = await self._client.get(f"addons/{addon}/options/config")
        return result.data

    async def rebuild_addon(
        self,
        addon: str,
        options: AddonsRebuild | None = None,
    ) -> None:
        """Rebuild an addon (only available for local addons built from source)."""
        await self._client.post(
            f"addons/{addon}/rebuild",
            json=options.to_dict() if options else None,
        )

    async def write_addon_stdin(self, addon: str, stdin: bytes) -> None:
        """Write to stdin of an addon (if supported by addon)."""
        await self._client.post(f"addons/{addon}/stdin", data=stdin)

    async def set_addon_security(
        self, addon: str, options: AddonsSecurityOptions
    ) -> None:
        """Set security options for addon."""
        await self._client.post(f"addons/{addon}/security", json=options.to_dict())

    async def addon_stats(self, addon: str) -> AddonsStats:
        """Get stats for addon."""
        result = await self._client.get(f"addons/{addon}/stats")
        return AddonsStats.from_dict(result.data)

    # Omitted for now - Log endpoints
